#
# ptree -- An xml.etree implementation with parent pointers.
#
# --------------------------------------------------------------------
# The ptree package is
#
# Copyright (c) 2007 by Edward Loper
#
# By obtaining, using, and/or copying this software and/or its
# associated documentation, you agree that you have read, understood,
# and will comply with the following terms and conditions:
#
# Permission to use, copy, modify, and distribute this software and
# its associated documentation for any purpose and without fee is
# hereby granted, provided that the above copyright notice appears in
# all copies, and that both that copyright notice and this permission
# notice appear in supporting documentation.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN
# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
# WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
# --------------------------------------------------------------------

"""
Two drop-in replacements for xml.etree.ElementTree with elements that
automatically keep track of parent pointers.  This package defines two
objects, each of which provide the same interface as the
xml.etree.ElementTree module:

  - ptree.ParentedTree: Each element keeps track of a single parent
    pointer, returned by the element's `parent()` method.  Elements
    may have at most one parent; i.e., subtrees may not be shared.
    Any attempt to do use a single element as a child for multiple
    parents will generate a ValueError.

  - ptree.MultiParentedTree: Each element keeps track of a list of
    parent pointers, returned by the element's `parents()` method.
    Elements may have zero or more parients; i.e., subtrees may be
    shraed.  If a single Element is used as multiple children of the
    same parent, then that parent will appear multiple times in the
    parents list.

Aside from addition of parent pointers, these two objects should act
identically to xml.etree.ElementTree.

Note: Mixing of etree implementations is not supported.  I.e., you
should never construct a tree that combines elements from ParentedTree
with elements from MultiParentedTree, or elements from either of these
implementations with elements from any other implementation.  Doing so
may result in incorrect parent pointers and ValueError exceptions.
"""

__all__ = ['ParentedTree', 'MultiParentedTree',
           'etreeimpl', 'elements']

from etreeimpl import ElementTreeImplementation
from elements import *

ParentedTree = ElementTreeImplementation(_ParentedElement)
MultiParentedTree = ElementTreeImplementation(_MultiParentedElement)

# Pretend they're modules
import sys
sys.modules['ptree.ParentedTree'] = ParentedTree
sys.modules['ptree.MultiParentedTree'] = MultiParentedTree
