#
# ptree.elements -- Element implementations for (multi)parented trees
#
# --------------------------------------------------------------------
# The ptree package is
#
# Copyright (c) 2007 by Edward Loper
#
# By obtaining, using, and/or copying this software and/or its
# associated documentation, you agree that you have read, understood,
# and will comply with the following terms and conditions:
#
# Permission to use, copy, modify, and distribute this software and
# its associated documentation for any purpose and without fee is
# hereby granted, provided that the above copyright notice appears in
# all copies, and that both that copyright notice and this permission
# notice appear in supporting documentation.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN
# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
# WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
# --------------------------------------------------------------------

"""
etree Element implementations for use with (multi)parented trees.
"""

from xml.etree import ElementTree as ET

__all__ = ['_ParentedElement', '_MultiParentedElement',
           'is_parented_element', 'is_multiparented_element']

######################################################################
# Parented Trees
######################################################################

class _AbstractParentedElement(ET._ElementInterface):
    """
    An abstract base class for (multi)parented element tree Elements.
    
      - Whenever a new child is added, L{_setparent} is called,
        which should update that child's parent pointer to point
        at self.
        
      - Whenever a child is removed, L{_delparent} is called, which
        should remove the child's parent pointer to self.
    """
    def __repr__(self):
        return "<%s %s at %x>" % (self.__class__.__name__,
                                  self.tag, id(self))

    #////////////////////////////////////////////////////////////
    # Parent management
    #////////////////////////////////////////////////////////////

    def _setparent(self, child):
        """
        Update C{child}'s parent pointer to point to self.
        """
        raise AssertionError, 'Abstract base class'
    
    def _delparent(self, child):
        """
        Remove self from C{child}'s parent pointer.
        """
        raise AssertionError, 'Abstract base class'

    #////////////////////////////////////////////////////////////
    # Methods that add/remove children
    #////////////////////////////////////////////////////////////
    # Every method that adds or removes a child must make
    # appropriate calls to _setparent() and _delparent().
    
    def __delitem__(self, index):
        self._delparent(self[index])
        ET._ElementInterface.__delitem__(self, index)

    def __delslice__(self, start, stop):
        for index in range(start, stop): self._delparent(self[index])
        ET._ElementInterface.__delslice__(self, start, stop)

    def __setitem__(self, index, element):
        self._delparent(self[index])
        self._setparent(element)
        ET._ElementInterface.__setitem__(self, index, element)

    def __setslice__(self, start, stop, elements):
        for index in range(start, stop): self._delparent(self[index])
        for val in elements: self._setparent(val)
        ET._ElementInterface.__setslice__(self, start, stop, elements)

    def append(self, element):
        self._setparent(element)
        ET._ElementInterface.append(self, element)

    def extend(self, elements):
        for val in elements: self._setparent(val)
        ET._ElementInterface.extend(self, elements)

    def insert(self, index, element):
        self._setparent(element)
        ET._ElementInterface.insert(self, index, element)

    def pop(self):
        self._delparent(self[-1])
        return ET._ElementInterface.pop(self, )

    def remove(self, element):
        index = self.index(element)
        self._delparent(self[index])
        ET._ElementInterface.remove(self, element)
    

class _ParentedElement(_AbstractParentedElement):
    """
    A specialized version of etree.ElementTree.Element that keeps
    track of a single parent pointer per element.

    Each _ParentedElement may have at most one parent.  In particular,
    subtrees may not be shared.  Any attempt to reuse a single
    _ParentedElement as a child of more than one parent (or as
    multiple children of the same parent) will cause a ValueError
    exception to be raised.
    
    _ParentedElements should never be used in the same tree as other
    Element implementation classes.  Mixing Element implementations
    may result in incorrect parent pointers and in C{ValueError}
    exceptions.
    """
    def __init__(self, tag, attrib):
        ET._ElementInterface.__init__(self, tag, attrib)
        self._parent = None

    def parent(self):
        return self._parent
        
    def _setparent(self, element):
        assert is_parented_element(element)
        if element._parent is not None:
            raise ValueError, '%r already has a parent' % element
        element._parent = self

    def _delparent(self, element):
        assert is_parented_element(element)
        assert element._parent == self
        element._parent = None

    def makeelement(self, tag, attrib):
        return _ParentedElement(tag, attrib)

def is_parented_element(element):
    return ET.iselement(element) and hasattr(element, '_parent')

class _MultiParentedElement(_AbstractParentedElement):
    """
    A specialized version of etree.ElementTree.Element that keeps
    track of a list of parent pointers for each element.

    Each _ParentedElement may have zero or more parents.  In
    particular, subtrees may be shared.  If a single
    _MultiParentedElement is used as multiple children of the same
    parent, then that parent will appear multiple times in the parents
    list.
    
    _MultiParentedElements should never be used in the same tree as
    other Element implementation classes.  Mixing Element
    implementations may result in incorrect parent pointers and in
    C{ValueError} exceptions.
    """
    def __init__(self, tag, attrib):
        ET._ElementInterface.__init__(self, tag, attrib)
        self._parents = []

    def parents(self):
        return tuple(self._parents)
        
    def _setparent(self, element):
        assert is_multiparented_element(element)
        element._parent.append(self)

    def _delparent(self, element):
        assert is_multiparented_element(element)
        assert self in element._parents
        element._parents.remove(self)

    def makeelement(self, tag, attrib):
        return _ParentedElement(tag, attrib)

def is_multiparented_element(element):
    return ET.iselement(element) and hasattr(element, '_parents')

