#
# ptree.etreeimpl -- Class for defining new etree implementations
#
# --------------------------------------------------------------------
# The ptree package is
#
# Copyright (c) 2007 by Edward Loper
#
# By obtaining, using, and/or copying this software and/or its
# associated documentation, you agree that you have read, understood,
# and will comply with the following terms and conditions:
#
# Permission to use, copy, modify, and distribute this software and
# its associated documentation for any purpose and without fee is
# hereby granted, provided that the above copyright notice appears in
# all copies, and that both that copyright notice and this permission
# notice appear in supporting documentation.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN
# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
# WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
# --------------------------------------------------------------------

__all__ = ['ElementTreeImplementation']

from xml.etree import ElementTree as ET

class ElementTreeImplementation(object):
    """
    Instances of this class can be used as drop-in replacements for
    the xml.etree.ElementTree module.
    """
    def __init__(self, ElementClass):
        self._Element = ElementClass

    def Element(self, tag, attrib={}, **extra):
        attrib = attrib.copy()
        attrib.update(extra)
        return self._Element(tag, attrib)

    ##################################################################
    # Subclasses w/ new constructors
    #-----------------------------------------------------------------
    # These classes needed to have their constructer overridden, to
    # change the default values of tree-building functions to point
    # to the versions in *this file, not in ElementTree.

    def iterparse(self, source, events=None):
        # [XXX] We can't do this without poking around in other
        # people's internals!
        iterator = ET.iterparse(source, events)
        iterator._parser._target = TreeBuilder()
        return iterator

    ##################################################################
    # No dependencies on Element class
    #-----------------------------------------------------------------
    # These functions & classes do not depend, directly or indirectly,
    # on the class used to implement elements; so we can just copy
    # them.

    SubElement = staticmethod(ET.SubElement)
    QName = ET.QName
    iselement = staticmethod(ET.iselement)
    dump = staticmethod(ET.dump)
    tostring= staticmethod(ET.tostring)
    
    ##################################################################
    # Modified Constructor Defaults
    #-----------------------------------------------------------------
    # These classes have default constructor parameter that depend on
    # the class used to implement elements; so we wrap them in
    # functions that provide new defaults.
    
    def TreeBuilder(self, element_factory=None):
        if element_factory is None:
            element_factory = self._Element
        return ET.TreeBuilder(element_factory)

    def XMLTreeBuilder(self, html=0, target=None):
        if target is None:
            target = self.TreeBuilder()
        return ET.XMLTreeBuilder(html, target)

    XMLParser = XMLTreeBuilder

    ##################################################################
    # Modified Method Defaults
    #-----------------------------------------------------------------
    # The ElementTree class has a method parameter whose default value
    # depends on the class used to implement elements; so we replace
    # it with a new subclass (_ElementTree) that stores a private
    # pointer back to the ElementTreeImplementation instance; and uses it
    # to construct an appropriate default for the method parameter.

    def ElementTree(self, element=None, file=None):
        return self._ElementTree(self, element, file)
    class _ElementTree(ET.ElementTree):
        def __init__(self, etbase, element, file):
            self.__default_parser_class = etbase.XMLTreeBuilder
            ET.ElementTree.__init__(self, element, file)
        def parse(self, source, parser=None):
            if not parser:
                parser = self.__default_parser_class()
            ET.ElementTree.parse(self, source, parser)

    ##################################################################
    # Indirect dependencies on Element class
    #-----------------------------------------------------------------
    # These functions depend indirectly on the class used to implement
    # elements; so we need to redefine them.  Each method in this
    # section is copied verbatim from etree/ElementTree.py, with the
    # following exceptions: a 'self' parameter is added.  And global
    # variables that depend on the Element class are replaced with
    # local versions.  E.g., "XMLTreeBuilder" is replaced with
    # "self.XMLTreeBuilder".
    
    def Comment(self, text=None):
        element = self.Element(Comment)
        element.text = text
        return element
    
    def ProcessingInstruction(self, target, text=None):
        element = self.Element(self.ProcessingInstruction)
        element.text = target
        if text:
            element.text = element.text + " " + text
        return element

    PI = ProcessingInstruction

    def parse(self, source, parser=None):
        tree = self.ElementTree()
        tree.parse(source, parser)
        return tree
    
    def XML(self, text):
        parser = self.XMLTreeBuilder()
        parser.feed(text)
        return parser.close()
    
    def XMLID(self, text):
        parser = self.XMLTreeBuilder()
        parser.feed(text)
        tree = parser.close()
        ids = {}
        for elem in tree.getiterator():
            id = elem.get("id")
            if id:
                ids[id] = elem
        return tree, ids
    
    fromstring = XML

